@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+// SPDX-FileCopyrightText: 2019 - 2023 UnionTech Software Technology Co., Ltd.
 //
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
@@ -31,6 +31,7 @@
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QProcess>
+#include <DConfig>
 #endif
 #include <QDir>
 #include <QLockFile>
@@ -106,6 +107,9 @@ Q_GLOBAL_STATIC(DFontManager, _globalFM)
 
 #define WINDOW_THEME_KEY "_d_platform_theme"
 
+#define APP_THEME_TYPE "themeType"
+Q_GLOBAL_STATIC_WITH_ARGS(DTK_CORE_NAMESPACE::DConfig, _d_dconfig, ("org.deepin.dtk.preference"));
+
 /*!
  @private
  */
@@ -117,11 +121,17 @@ public:
     {
         // 临时存储一个无效的指针值, 用于此处条件变量的竞争
         if (m_helper.testAndSetRelaxed(nullptr, INVALID_HELPER)) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+            m_helper.storeRelaxed(creator());
+            m_helper.loadRelaxed()->initialize();
+        }
+        return m_helper.loadRelaxed();
+#else
             m_helper.store(creator());
             m_helper.load()->initialize();
         }
-
         return m_helper.load();
+#endif
     }
 
     inline void clear()
@@ -251,7 +261,7 @@ void DGuiApplicationHelperPrivate::staticInitApplication()
     if (!_globalHelper.exists())
         return;
 
-    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load())
+    if (DGuiApplicationHelper *helper = _globalHelper->helper())
         helper->d_func()->initApplication(qGuiApp);
 }
 
@@ -316,11 +326,7 @@ void DGuiApplicationHelperPrivate::_q_initApplicationTheme(bool notifyChange)
 void DGuiApplicationHelperPrivate::notifyAppThemeChanged()
 {
     D_Q(DGuiApplicationHelper);
-
-    QWindowSystemInterfacePrivate::ThemeChangeEvent event(nullptr);
-    // 此事件会促使QGuiApplication重新从QPlatformTheme中获取系统级别的QPalette.
-    // 而在deepin平台下, 系统级别的QPalette来源自 \a applicationPalette()
-    QGuiApplicationPrivate::processThemeChanged(&event);
+    notifyAppThemeChangedByEvent();
     // 通知主题类型发生变化, 此处可能存在误报的行为, 不过不应该对此做额外的约束
     // 此信号的行为应当等价于 applicationPaletteChanged
     Q_EMIT q->themeTypeChanged(q->themeType());
@@ -328,11 +334,76 @@ void DGuiApplicationHelperPrivate::notifyAppThemeChanged()
     Q_EMIT q->applicationPaletteChanged();
 }
 
+void DGuiApplicationHelperPrivate::notifyAppThemeChangedByEvent()
+{
+    QWindowSystemInterfacePrivate::ThemeChangeEvent event(nullptr);
+    // 此事件会促使QGuiApplication重新从QPlatformTheme中获取系统级别的QPalette.
+    // 而在deepin平台下, 系统级别的QPalette来源自 \a applicationPalette()
+    QGuiApplicationPrivate::processThemeChanged(&event);
+}
+
 bool DGuiApplicationHelperPrivate::isCustomPalette() const
 {
     return appPalette || paletteType != DGuiApplicationHelper::UnknownType;
 }
 
+void DGuiApplicationHelperPrivate::setPaletteType(DGuiApplicationHelper::ColorType ct, bool emitSignal)
+{
+    if (paletteType == ct)
+        return;
+
+    if (qGuiApp && qGuiApp->testAttribute(Qt::AA_SetPalette))
+        qWarning() << "DGuiApplicationHelper: Plase check 'QGuiApplication::setPalette',"
+                      " Don't use it on DTK application.";
+
+    paletteType = ct;
+
+    if (!emitSignal) {
+        notifyAppThemeChangedByEvent();
+        return;
+    }
+
+    // 如果未固定调色板, 则paletteType的变化可能会导致调色板改变, 应当通知程序更新数据
+    if (!appPalette)
+        notifyAppThemeChanged();
+
+    D_Q(DGuiApplicationHelper);
+    Q_EMIT q->paletteTypeChanged(paletteType);
+}
+
+void DGuiApplicationHelperPrivate::initPaletteType() const
+{
+    DCORE_USE_NAMESPACE
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::DontSaveApplicationTheme))
+        return;
+
+    if (_d_dconfig.exists())
+        return;
+
+    auto applyThemeType = [this](bool emitSignal){
+        int ct = _d_dconfig->value(APP_THEME_TYPE, DGuiApplicationHelper::UnknownType).toInt();
+        if (ct > DGuiApplicationHelper::DarkType || ct < DGuiApplicationHelper::UnknownType)
+            ct = DGuiApplicationHelper::UnknownType;
+
+        const_cast<DGuiApplicationHelperPrivate *>(this)->setPaletteType(DGuiApplicationHelper::ColorType(ct), emitSignal);
+    };
+
+    applyThemeType(false);
+
+    auto conn = QObject::connect(_d_dconfig, &DConfig::valueChanged, _d_dconfig, [applyThemeType](const QString &key){
+        if (key != APP_THEME_TYPE)
+            return;
+
+        applyThemeType(true);
+    });
+
+    QObject::connect(qGuiApp, &QGuiApplication::aboutToQuit, _d_dconfig, [conn](){
+        QObject::disconnect(conn);
+        int paletteType = DGuiApplicationHelper::instance()->paletteType();
+        _d_dconfig->setValue(APP_THEME_TYPE, paletteType);
+    });
+}
+
 void DGuiApplicationHelperPrivate::_q_sizeModeChanged(int mode)
 {
     D_Q(DGuiApplicationHelper);
@@ -468,6 +539,43 @@ DGuiApplicationHelper::~DGuiApplicationHelper()
     _globalHelper->m_helper = nullptr;
 }
 
+static inline QColor adjustHSLColor(const QColor &base, qint8 hueFloat, qint8 saturationFloat,
+                                    qint8 lightnessFloat, qint8 alphaFloat = 0)
+{
+    if (Q_LIKELY(hueFloat || saturationFloat || lightnessFloat || alphaFloat)) {
+        // 按HSL格式调整
+        int H, S, L, A;
+        base.getHsl(&H, &S, &L, &A);
+
+        H = H > 0 ? adjustColorValue(H, hueFloat, 359) : H;
+        S = adjustColorValue(S, saturationFloat);
+        L = adjustColorValue(L, lightnessFloat);
+
+        return QColor::fromHsl(H, S, L, A);
+    }
+
+    return base;
+}
+
+static inline QColor adjustRGBColor(const QColor &base, qint8 redFloat, qint8 greenFloat,
+                                    qint8 blueFloat, qint8 alphaFloat = 0)
+{
+    if (Q_LIKELY(redFloat || greenFloat || blueFloat || alphaFloat)) {
+        // 按RGB格式调整
+        int R, G, B, A;
+        base.getRgb(&R, &G, &B, &A);
+
+        R = adjustColorValue(R, redFloat);
+        G = adjustColorValue(G, greenFloat);
+        B = adjustColorValue(B, blueFloat);
+        A = adjustColorValue(A, alphaFloat);
+
+        return QColor(R, G, B, A);
+    }
+
+    return base;
+}
+
 /*!
   \brief 调整颜色.
 
@@ -487,28 +595,18 @@ QColor DGuiApplicationHelper::adjustColor(const QColor &base,
                                           qint8 hueFloat, qint8 saturationFloat, qint8 lightnessFloat,
                                           qint8 redFloat, qint8 greenFloat, qint8 blueFloat, qint8 alphaFloat)
 {
-    // 按HSL格式调整
-    int H, S, L, A;
-    base.getHsl(&H, &S, &L, &A);
+    if (Q_UNLIKELY(!base.isValid()))
+        return base;
 
-    H = H > 0 ? adjustColorValue(H, hueFloat, 359) : H;
-    S = adjustColorValue(S, saturationFloat);
-    L = adjustColorValue(L, lightnessFloat);
-    A = adjustColorValue(A, alphaFloat);
+    // First do the adjust of spec of this color, Avoid precision loss caused by color spec conversion.
+    if (Q_UNLIKELY(base.spec() == QColor::Hsl)) {
+        QColor newColor = adjustHSLColor(base, hueFloat, saturationFloat, lightnessFloat, alphaFloat);
+        return adjustRGBColor(newColor, redFloat, greenFloat, blueFloat);
+    }
 
-    QColor new_color = QColor::fromHsl(H, S, L, A);
-
-    // 按RGB格式调整
-    int R, G, B;
-    new_color.getRgb(&R, &G, &B);
-
-    R = adjustColorValue(R, redFloat);
-    G = adjustColorValue(G, greenFloat);
-    B = adjustColorValue(B, blueFloat);
-
-    new_color.setRgb(R, G, B, A);
-
-    return new_color;
+    // For other specs, first do RGB adjust
+    QColor newColor = adjustRGBColor(base, redFloat, greenFloat, blueFloat, alphaFloat);
+    return adjustHSLColor(newColor, hueFloat, saturationFloat, lightnessFloat);
 }
 
 /*!
@@ -575,15 +673,15 @@ QColor DGuiApplicationHelper::blendColor(const QColor &substrate, const QColor &
 }
 
 static QColor light_qpalette[QPalette::NColorRoles] {
-    QColor("#414d68"),                  //WindowText
+    QColor(0, 0, 0, 0.7 * 255),         //WindowText
     QColor("#e5e5e5"),                  //Button
     QColor("#e6e6e6"),                  //Light
     QColor("#e5e5e5"),                  //Midlight
     QColor("#e3e3e3"),                  //Dark
     QColor("#e4e4e4"),                  //Mid
-    QColor("#414d68"),                  //Text
+    QColor(0, 0, 0, 0.7 * 255),         //Text
     Qt::black,                          //BrightText
-    QColor("#414d68"),                  //ButtonText
+    QColor(0, 0, 0, 0.7 * 255),         //ButtonText
     Qt::white,                          //Base
     QColor("#f8f8f8"),                  //Window
     QColor(0, 0, 0, 0.05 * 255),        //Shadow
@@ -594,19 +692,19 @@ static QColor light_qpalette[QPalette::NColorRoles] {
     QColor(0, 0, 0, 0.03 * 255),        //AlternateBase
     Qt::white,                          //NoRole
     QColor(255, 255, 255, 0.8 * 255),   //ToolTipBase
-    Qt::black                           //ToolTipText
+    QColor(0, 0, 0, 0.85 * 255)         //ToolTipText
 };
 
 static QColor dark_qpalette[QPalette::NColorRoles] {
-    QColor("#c0c6d4"),                  //WindowText
+    QColor(255, 255, 255, 0.7 * 255),   //WindowText
     QColor("#444444"),                  //Button
     QColor("#484848"),                  //Light
     QColor("#474747"),                  //Midlight
     QColor("#414141"),                  //Dark
     QColor("#434343"),                  //Mid
-    QColor("#c0c6d4"),                  //Text
+    QColor(255, 255, 255, 0.7 * 255),   //Text
     Qt::white,                          //BrightText
-    QColor("#c0c6d4"),                  //ButtonText
+    QColor(255, 255, 255, 0.7 * 255),   //ButtonText
     QColor("#282828"),                  //Base
     QColor("#252525"),                  //Window
     QColor(0, 0, 0, 0.05 * 255),        //Shadow
@@ -617,14 +715,14 @@ static QColor dark_qpalette[QPalette::NColorRoles] {
     QColor(0, 0, 0, 0.05 * 255),        //AlternateBase
     Qt::black,                          //NoRole
     QColor(45, 45, 45, 0.8 * 255),      //ToolTipBase
-    QColor("#c0c6d4")                   //ToolTipText
+    QColor(255, 255, 255, 0.85 * 255)   //ToolTipText
 };
 
 static QColor light_dpalette[DPalette::NColorTypes] {
     QColor(),                       //NoType
     QColor(0, 0, 0, 255 * 0.03),    //ItemBackground
-    QColor("#001A2E"),              //TextTitle
-    QColor("#526A7F"),              //TextTips
+    QColor(0, 0, 0, 0.85 * 255),    //TextTitle
+    QColor(0, 0, 0, 0.6 * 255),     //TextTips
     QColor("#FF5736"),              //TextWarning
     Qt::white,                      //TextLively
     QColor("#0081FF"),              //LightLively
@@ -638,8 +736,8 @@ static QColor light_dpalette[DPalette::NColorTypes] {
 static QColor dark_dpalette[DPalette::NColorTypes] {
     QColor(),                           //NoType
     QColor(255, 255, 255, 255 * 0.05),  //ItemBackground
-    QColor("#C0C6D4"),                  //TextTitle
-    QColor("#6D7C88"),                  //TextTips
+    QColor(255, 255, 255, 0.85 * 255),  //TextTitle
+    QColor(255, 255, 255, 0.6 * 255),   //TextTips
     QColor("#E43F2E"),                  //TextWarning
     Qt::white,                          //TextLively
     QColor("#0059d2"),                  //LightLively
@@ -1119,13 +1217,21 @@ void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
         qWarning() << "DGuiApplicationHelper: Plase check 'QGuiApplication::setPalette', Don't use it on DTK application.";
     }
 
+    auto resolve = [](const DPalette &palette) ->bool {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    return palette.resolveMask();
+#else
+    return palette.resolve();
+#endif
+    };
+
     if (d->appPalette) {
-        if (palette.resolve()) {
+        if (resolve(palette)) {
             *d->appPalette = palette;
         } else {
             d->appPalette.reset();
         }
-    } else if (palette.resolve()) {
+    } else if (resolve(palette)) {
         d->appPalette.reset(new DPalette(palette));
     } else {
         return;
@@ -1221,7 +1327,7 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QColor
  */
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QPalette &palette)
 {
-    return toColorType(palette.background().color());
+    return toColorType(palette.window().color());
 }
 
 /*!
@@ -1239,6 +1345,7 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::themeType() const
 {
     D_DC(DGuiApplicationHelper);
 
+    d->initPaletteType();
     if (d->paletteType != UnknownType) {
         return d->paletteType;
     }
@@ -1258,6 +1365,8 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::themeType() const
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::paletteType() const
 {
     D_DC(DGuiApplicationHelper);
+
+    d->initPaletteType();
     return d->paletteType;
 }
 
@@ -1373,8 +1482,8 @@ bool DGuiApplicationHelper::setSingleInstance(const QString &key, DGuiApplicatio
                 qCInfo(dgAppHelper) << "New instance: pid=" << pid << "arguments=" << arguments;
 
                 // 通知新进程的信息
-                if (_globalHelper.exists() && _globalHelper->m_helper.load())
-                    Q_EMIT _globalHelper->m_helper.load()->newProcessInstance(pid, arguments);
+                if (_globalHelper.exists() && _globalHelper->helper())
+                    Q_EMIT _globalHelper->helper()->newProcessInstance(pid, arguments);
             });
 
             instance->flush(); //发送数据给新的实例
@@ -1445,7 +1554,8 @@ bool DGuiApplicationHelper::hasUserManual() const
         return hasManual;
 
     QDBusConnection conn = QDBusConnection::sessionBus();
-    if (!conn.isConnected()) {
+    if (!conn.isConnected() ||
+        !conn.interface()->isServiceRegistered("com.deepin.Manual.Search")) {
         static LoadManualServiceWorker *manualWorker = new LoadManualServiceWorker;
         manualWorker->checkManualServiceWakeUp();
 
@@ -1493,7 +1603,12 @@ bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<
     QStringList missingQmfiles;
     for (const auto &locale : localeFallback) {
         QStringList translateFilenames {QString("%1_%2").arg(fileName).arg(locale.name())};
-        const QStringList parseLocalNameList = locale.name().split("_", QString::SkipEmptyParts);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+        auto behavior = Qt::SkipEmptyParts;
+#else
+        auto behavior = QString::SkipEmptyParts;
+#endif
+        const QStringList parseLocalNameList = locale.name().split("_", behavior);
         if (parseLocalNameList.length() > 0)
             translateFilenames << QString("%1_%2").arg(fileName).arg(parseLocalNameList.at(0));
 
@@ -1574,8 +1689,10 @@ bool DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::Attribute attri
             return false;
         }
         return QString(typeid(*QGuiApplicationPrivate::platform_theme).name()).contains("QDeepinTheme");
-    case IsDeepinEnvironment:
-        return QGuiApplicationPrivate::instance()->platformIntegration()->services()->desktopEnvironment().toLower().contains("deepin");
+    case IsDeepinEnvironment: {
+        const auto &de = QGuiApplicationPrivate::instance()->platformIntegration()->services()->desktopEnvironment();
+        return de.toLower().contains("deepin") || !de.compare("DDE");
+    }
     case IsSpecialEffectsEnvironment: {
         return qgetenv("DTK_DISABLED_SPECIAL_EFFECTS").toInt() != 1;
     }
@@ -1607,21 +1724,15 @@ void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeT
  */
 void DGuiApplicationHelper::setPaletteType(DGuiApplicationHelper::ColorType paletteType)
 {
-    D_D(DGuiApplicationHelper);
-
-    if (d->paletteType == paletteType)
+    if (!qApp) {
+        qWarning() << "Can't call `DGuiApplicationHelper::setPaletteType` before QCoreApplication constructed.";
         return;
-
-    if (qGuiApp && qGuiApp->testAttribute(Qt::AA_SetPalette)) {
-        qWarning() << "DGuiApplicationHelper: Plase check 'QGuiApplication::setPalette', Don't use it on DTK application.";
     }
 
-    d->paletteType = paletteType;
-    // 如果未固定调色板, 则paletteType的变化可能会导致调色板改变, 应当通知程序更新数据
-    if (!d->appPalette)
-        d->notifyAppThemeChanged();
+    D_D(DGuiApplicationHelper);
 
-    Q_EMIT paletteTypeChanged(paletteType);
+    d->initPaletteType();
+    d->setPaletteType(paletteType, true);
 }
 
 /*!
